// TAEKWONDO : réception pattern sur ESP32 (WPB109) par le Bluetooth 
// Pattern envoyé par l'appli Android "TAEKWONDO"
// A compiler pour carte "ESP32 Dev Module"
// avec affichage messages recus sur écran Oled 128x64 pour tests
// --------------------------------------------------------------

//-----------led témoin erreur
#define lederr 33

//-----------timer hardware
hw_timer_t * timerhard = NULL;
volatile bool finmvt = false;

// ----------- buzzer
#include "EasyBuzzer.h"       // Attention : utilise le channel ledcpwm 0
#define BUZZ 19

// ----------- bouton
#include <pushbuttoncommand.h>
#define BOUT 23
#define nb_boutons      1    // nbre de boutons utilisés
#define antirebond      2    // durée antirebond
#define doubleclic      8   // durée double clic
#define appuilong       20   // durée appui long
byte bouton_val = 0;         // valeur retournee par le bouton
byte bouton_ret = 0;         // code retour de l'appui bouton 1=clic, 2 = double clic, 3=appui long
byte savbout = 0;            // mémoriser appui bouton
PushButtonCommand poussebouton;

//---------- Ecran OLed
#include <Wire.h>
#include <U8g2lib.h>
#define SDA 21 (sur esp32)
#define SCL 22 (sur esp32)
U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0);  
// liste des écrans à https://github.com/olikraus/u8g2/wiki/u8g2setupcpp

//------------ bluetooth
#include "BluetoothSerial.h"  
BluetoothSerial SerialBT;
char incoming;
String message = "";
String savmess = "";
bool etatconn = false;
String nomctlr =             "TAEKWONDO.01";  // à modifier pour chaque appareil

//------------Magnetomètre + gyroscope
#define CMPS12 0x60  // Address of CMPS12 shifted right one bit for arduino wire library
unsigned char high_byte, low_byte;
unsigned int azimut;            // en degrés
int accelz;

//------------pour calcul rotations :
int azimdep = 0;
int azimfin = 0;
bool rot = false;
const int seuildep = 50;
const int seuilrot = 1500;
int sommaccel = 0;          // somme algébrique des accélérations
String sensrot = "";
int anglerot = 0;
bool nouvtest = false;

//------------pour analyse message :
int i,j = 0;
String nompatt = "";
String niveau = "";
String chaine = "";
int durmvt,angmvt,durpatt = 0;
bool nouvanal,finanal = false;

//------------pour contrôle exécution du pattern :
int rotmvt,nberrot = 0;
bool nouvctrl,finctrl = false;
int rotmin,rotmax = 0;
const int margerr = 15;       // marge d'erreur sur rotation
//------------------------------------------
void IRAM_ATTR timefin(){
  finmvt = true;
}
//------------------------------------------
void setup() {
//  Serial.begin(115200); 
  // utilisation du timer 0 avec un prescaler de 80 car horloge de ESP32 = 80 Mhz  
  // donc 1 tic dure 80MHZ/80 = 1 µs   donc 1 seconde = 1.000.000 µs 
  timerhard = timerBegin(0, 80, true);
  timerAttachInterrupt(timerhard, &timefin, true);
  pinMode(lederr, OUTPUT);        // led erreur de rotation
  digitalWrite(lederr,LOW);
  EasyBuzzer.update();         // correction bug dans lib EasyBuzzer
  EasyBuzzer.setPin(BUZZ);     //  Pin buzzer 
  SerialBT.register_callback(callback);
  SerialBT.begin(nomctlr);
  pinMode(BOUT,INPUT_PULLUP);
  poussebouton.setup(nb_boutons, antirebond, doubleclic, appuilong); 
  Wire.begin();               // Initialise le bus I2C
  Wire.setClock(400000);      // vitesse transmission I2C à 400 KHz
  u8g2.begin(); 
  u8g2.enableUTF8Print();     // permettre les caractères accentués                                     
  affichage1();
}
//------------------------------------------
void Appui_Bouton() {
  if(bouton_val == 0)   bouton_val = poussebouton.loop(1, !digitalRead(BOUT));
  switch(bouton_val) {
  case PUSH_CMD_CLICK:
    bouton_ret = 1;     // clic 
    break;
  case PUSH_CMD_DOUBLE_CLICK:
    bouton_ret = 2;     // double clic
    break;
  case PUSH_CMD_LONG_CLICK:
    bouton_ret = 3;     // clic long
    break;
  case PUSH_CMD_LONG_PUSH:     
    bouton_ret = 4;      // appui maintenu
    break;
  }
  if (bouton_val != 0)  bouton_val = 0; 
  delay(10);
}
//------------------------------------------
void callback(esp_spp_cb_event_t event, esp_spp_cb_param_t *param) {
  if(event == ESP_SPP_SRV_OPEN_EVT){
    etatconn = true;     //  client Bluetooth connecté
    savmess = ""; 
  }
  if(event == ESP_SPP_CLOSE_EVT ){
    etatconn = false;    //  client Bluetooth déconnecté
  }
}
//------------------------------------------
void recptmess() {
  while (SerialBT.available()) { 
    incoming = SerialBT.read();
//    Serial.println(incoming); 
    message += incoming;    
    delay(10); 
  }
}
//------------------------------------------
void sonner(int freq, int duree) {
  EasyBuzzer.singleBeep(freq,duree);      // faire sonner la cible
  delay(duree);
  EasyBuzzer.stopBeep();
}
//------------------------------------------
 
void loop() {
  Appui_Bouton();
  if (bouton_ret != 0) {
    savbout = bouton_ret;
    nouvtest = true;
    nouvanal = true;
    nouvctrl = true;
    anglerot = 0;
    sensrot = "";
    azimfin = 0;
    bouton_ret = 0; 
    digitalWrite(lederr,LOW);
  }
  switch(savbout) {
    case 0:                // début de programme avant tout clic
      message = "";
      recptmess();
      if (message != "") {
        savmess = message;
        SerialBT.disconnect();
      }
      affichage1();
      break;
    case 1:                 // Clic
      affichage3();
      delay(4000);
      savbout = 0;
      nouvanal = true;  
      nouvctrl = true; 
      break;
    case 2:                 // Double clic;
      testrota(); 
      affichage2();  
      break;
    case 3:                 // Clic long
      controle(); 
      savbout = 0; 
      break; 
  }
}
//------------------------------------------------
